@@ -40,6 +40,9 @@ imxHasOpenMP <- function() .Call(hasOpenMP_wrapper)
 .onAttach <- function(libname, pkgname) {
 	if (!imxHasOpenMP()) {
 		packageStartupMessage("OpenMx is not compiled to take advantage of computers with multiple cores.")
+	} else if (Sys.getenv("OMP_NUM_THREADS") == "") {
+		packageStartupMessage(paste0("To take full advantage of multiple cores, use:\n",
+					     "  mxOption(NULL, 'Number of Threads', parallel::detectCores())"))
 	}
 	if (!is.na(match("package:expm", search()))) {
 		packageStartupMessage(paste("** Holy cannoli! You must be a pretty advanced and awesome user.",
