@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2015 The OpenMx Project
+#   Copyright 2007-2016 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -127,6 +127,24 @@ rms <- function(x, y){sqrt(mean((x-y)^2))}
 initialCond
 omxCheckTrue(rms(ssInitialCond, initialCond) < .1)
 
+#------------------------------------------------------------------------------
+# Example 1 continued
+#  Data generation and re-fitting
+
+new_data <- mxGenerateData(oscr, nrow(y))
+new_data$tim <- y[, 'tim']
+
+nosc <- mxModel(osc, mxData(new_data, 'raw'))
+noscr <- mxRun(nosc)
+
+# Parameters look close
+cbind(coef(oscr), coef(noscr))
+se <- summary(oscr)$parameters[,6]
+
+# Check that the old and new data estimates (data generated by sine+noise vs exact discrete)
+#  all with confidence intervals of one another.
+withinCI <- (coef(noscr) < (coef(oscr) + 1.5*se)) & (coef(noscr) > (coef(oscr) - 1.5*se))
+omxCheckEquals(withinCI, rep(TRUE, length(se)))
 
 
 #------------------------------------------------------------------------------
