@@ -37,6 +37,7 @@ m1 <- mxModel(m1, mxModel(
   mxPath(c("intercept", "slope"), arrows=2, values=1),
   mxPath("intercept", "slope", arrows=2, values=.25, labels="cov1")))
 
+m1$bySubject.S[1,1]$ubound <- 100
 m1$bySubject$fitfunction <- NULL
 
 omxCheckError(mxRun(m1), "Join mapping matrix sleep.Z must have 2 rows: 'Reaction' and 'DayEffect'")
@@ -62,7 +63,7 @@ m2 <- mxModel(model="sleep", type="RAM", manifestVars=c("Reaction"), latentVars
               mxData(type="raw", observed=sleepstudy, sort = FALSE),
               mxPath(c("one"), "Reaction"),
               mxPath(c("one"), "DayEffect", free=FALSE, labels="data.Days"),
-              mxPath("DayEffect", "Reaction"),
+              mxPath("DayEffect", "Reaction", values=1),
               mxPath(c("Reaction"), arrows=2, values=1))
 
 omxCheckError(mxModel(m2, mxPath('by_Subject.intercept', 'Reaction',
