@@ -269,6 +269,18 @@ omxMatrix* omxMatrixLookupFromState1(SEXP matrix, omxState* os) {
 	return os->getMatrixFromIndex(value);
 }
 
+omxMatrix* omxMatrixLookupFromStateByNumber(int matrix, omxState* os) {
+	omxMatrix* output = NULL;
+	if(matrix == NA_INTEGER){return NULL;}
+	if (matrix >= 0) {
+		output = os->algebraList[matrix];
+	} 
+	else {
+		output = os->matrixList[~matrix];
+	}
+	return output;
+}
+
 omxMatrix* omxNewAlgebraFromOperatorAndArgs(int opCode, omxMatrix* args[], int numArgs, omxState* os) {
 	
 	if(OMX_DEBUG) {mxLog("Generating new algebra from opcode %d (%s).", opCode, omxAlgebraSymbolTable[opCode].rName);}
