@@ -105,7 +105,8 @@ enum omxFFCompute {
 	FF_COMPUTE_INFO         = 1<<8,   // Fisher information
 	FF_COMPUTE_BESTFIT      = 1<<9,
 	FF_COMPUTE_STARTING     = 1<<10,   // for special hacks, not for routine use
-	FF_COMPUTE_INITIAL_FIT  = 1<<11    // omxInitialMatrixAlgebraCompute
+	FF_COMPUTE_INITIAL_FIT  = 1<<11,    // omxInitialMatrixAlgebraCompute
+	FF_COMPUTE_FINAL_FIT    = 1<<12
 };
 
 typedef struct omxMatrix omxMatrix;
