@@ -15,7 +15,7 @@
  */
  
  typedef struct {
-  omxMatrix *cov, *invcov, *means, *X, *logdetV_om, *cholV_fail_om;
+  omxMatrix *cov, *invcov, *means, *X, *logdetV_om, *cholV_fail_om, *origVdim_om;
   omxData *y, *data2;
   int alwaysComputeMeans, numcases2drop, cholquadX_fail;
   std::vector< int > dropcase;
@@ -30,6 +30,6 @@ void omxComputeGREMLExpectation(omxExpectation* ox, FitContext *fc, const char *
 void omxDestroyGREMLExpectation(omxExpectation* ox);
 void omxPopulateGREMLAttributes(omxExpectation *ox, SEXP algebra);
 void dropCasesAndEigenize(omxMatrix* om, Eigen::MatrixXd &em, int num2drop, std::vector< int > todrop, 
-	int symmetric);
+	int symmetric, int origDim);
 omxMatrix* omxGetGREMLExpectationComponent(omxExpectation* ox, const char* component);
 
