@@ -123,6 +123,11 @@ struct EigenMatrixAdaptor : Eigen::Map< Eigen::MatrixXd > {
 	  Eigen::Map< Eigen::MatrixXd >(omxMatrixDataColumnMajor(mat), mat->rows, mat->cols) {}
 };
 
+struct EigenArrayAdaptor : Eigen::Map< Eigen::ArrayXXd > {
+	EigenArrayAdaptor(omxMatrix *mat) :
+	  Eigen::Map< Eigen::ArrayXXd >(omxMatrixDataColumnMajor(mat), mat->rows, mat->cols) {}
+};
+
 struct EigenVectorAdaptor : Eigen::Map< Eigen::VectorXd > {
 	EigenVectorAdaptor(omxMatrix *mat) :
 	  Eigen::Map< Eigen::VectorXd >(mat->data, mat->rows * mat->cols) {}
