@@ -85,7 +85,7 @@ static void omxRowFitFunctionSingleIteration(omxFitFunction *localobj, omxFitFun
 	for(int row = rowbegin; row < data->rows && (row - rowbegin) < rowcount; row++) {
 		mxLogSetCurrentRow(row);
 
-		data->handleDefinitionVarList(localobj->matrix->currentState, row);
+		data->loadDefVars(localobj->matrix->currentState, row);
 
         // Populate data row
 		if (isContiguous) {
@@ -125,7 +125,7 @@ static void omxCallRowFitFunction(omxFitFunction *oo, int want, FitContext *fc)
 	// Requires: Data, means, covariances.
 
 	omxMatrix* objMatrix  = oo->matrix;
-	int numChildren = fc->childList.size();
+	int numChildren = fc? fc->childList.size() : 0;
 
     omxMatrix *reduceAlgebra;
 	omxData *data;
@@ -286,6 +286,8 @@ void omxInitRowFitFunction(omxFitFunction* oo) {
 
 	oo->computeFun = omxCallRowFitFunction;
 	oo->destructFun = omxDestroyRowFitFunction;
+	oo->canDuplicate = true;
+	oo->openmpUser = true;
 
 	oo->argStruct = (void*) newObj;
 }
